//---------------------------------------------------------------------------
// Borland BDS2006
// The DevExpress Library is used for the switch buttons.
// TMS Components (asynch32) is used for the com port.
#include <vcl.h>
#pragma hdrstop
#include <inifiles.hpp>

#include "Main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "VaClasses"
#pragma link "VaComm"
#pragma link "cxButtons"
#pragma link "cxLookAndFeelPainters"
#pragma resource "*.dfm"

#define BUTTON_TOP (10+(26* (i)))


TFrmMain* FrmMain;
//---------------------------------------------------------------------------
__fastcall TFrmMain::TFrmMain(TComponent* Owner)
: TForm(Owner)
{
  // Initialize the button variables	
  for ( int i = 0; i < 25; ++i )
    SBp[i] = NULL;
  // Get the ini file name
  AnsiString FileName = ChangeFileExt(ParamStr(0), ".ini");
  if ( !FileExists(FileName) )
	{
    Beep();
    AnsiString S = AnsiString("The file ")+FileName+AnsiString(" is missing.\nShall I create one?");
    S = S + "\n\nNOTE: You can edit this file to add or remove switch buttons.";
    if ( MessageDlg(S, mtWarning, TMsgDlgButtons() << mbYes<<mbNo, 0) == mrNo )
      exit(1);
		else
      CreateIniFile();
  }
  // Read or create the ini file
  TIniFile* Ini = new TIniFile(FileName);
	VaComm->DeviceName=Ini->ReadString("Serial", "ComPort", "COM1");
	String S = Ini->ReadString("Serial", "AltChar", "No");
	if(!CompareText("No", S) || !CompareText("false", S))
		MenuChar = '~';
	else
		MenuChar = '`';

	char ch[2];
	ch[1] = 0;
  buttonCount = 0;
//  Create the buttons based on the ini file
	for ( int i = 0;  i < 20; ++i )
	{
    ch[0] = i+'A';
    AnsiString AChar = ch;
    if ( Ini->ReadString("SwitchNames", AChar, "").Length() )
    {
      SBp[buttonCount] = new TcxButton(FrmMain);
      SBp[buttonCount]->Parent = FrmMain;
      SBp[buttonCount]->Font->Name = "Terminal";
      SBp[buttonCount]->Font->Size = 10;
      SBp[buttonCount]->Left = 20;
      SBp[buttonCount]->Top = BUTTON_TOP;
      SBp[buttonCount]->Caption = AChar;
      SBp[buttonCount]->Tag = i+'A';
      SBp[buttonCount]->OnClick = SpeedButtonClick;
      SBp[buttonCount]->Hint = "Press "+AChar+" or click this button to turn on the port"; 
      SBp[buttonCount]->TabStop = false;
      SBp[buttonCount]->Caption = AnsiString("&")+AChar + " - "+Ini->ReadString("SwitchNames", AChar, "");
      SBp[buttonCount]->Margin = 5;
      SBp[buttonCount]->LookAndFeel->Kind = lfOffice11;
      buttonCount++;
    }
  }
  delete Ini;

  // Put the captions on the buttons
  int length = 0;
  for ( int i = 0; i < buttonCount; ++i )
  {
    if ( length < SBp[i]->Caption.Length() )
    {
      length = SBp[i]->Caption.Length();
    }
  }
  for ( int i = 0; i < buttonCount; ++i )
    SBp[i]->Width = 18+length *8;

  ArrangeButtons();
  // Tell the application to start	
  PostMessage( this->Handle, CM_APPSTART, 0, 0 );
}
//---------------------------------------------------------------------------
// The Application is constructed and is now processing messages
//---------------------------------------------------------------------------
void __fastcall TFrmMain::AppStart( TMessage &Message )
{
  // Show the form
  Visible = true;
}
//---------------------------------------------------------------------------

void __fastcall TFrmMain::BtnExitClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
// A button is clicked. Determine which one and switch the port.
//---------------------------------------------------------------------------
void __fastcall TFrmMain::SpeedButtonClick(TObject *Sender)
{
  Screen->Cursor = crHourGlass;
  Application->ProcessMessages();
  TcxButton* SB;
	 
  SB = dynamic_cast<TcxButton*>(Sender);
  
  if ( SB )
  {
    VaComm->Baudrate=br9600;
    VaComm->Databits=db8;
    VaComm->Parity=paNone;
    VaComm->Stopbits=sb1;
    try
    {
      if ( !VaComm->Active() )
        VaComm->Open();
    }
    catch ( Exception &E )
    {
      Screen->Cursor = crDefault;
      Beep();
      Application->ShowException( &E);
      return;
    }
    if ( !VaComm->Active() )
    {
      Beep();
      Screen->Cursor = crDefault;
      ShowMessage("Cannot open "+VaComm->DeviceName);
      return;
    }
    // Enter the menu	
		VaComm->WriteChar(MenuChar);
		Sleep(500);
		VaComm->WriteChar(MenuChar);
		Sleep(50);
    // shut off all ports
    VaComm->WriteChar('w');
    Sleep(50);
    // Turn on the selected port
    VaComm->WriteChar(SB->Tag);
    Sleep(50);
    VaComm->WriteChar('x');
    VaComm->Close();
  }
  Screen->Cursor = crDefault;
  Application->ProcessMessages();
}
//---------------------------------------------------------------------------
// Handle keyboard shortcuts
//---------------------------------------------------------------------------
void __fastcall TFrmMain::FormKeyPress(TObject *Sender, char &Key)
{

  if ( (Key >= 'a' && Key <= 'z') || (Key >= 'A' && Key <= 'Z') )
  {
    AnsiString S = UpperCase(Key);
    char ch = S.c_str()[0];
    for ( int i = 0; i < buttonCount; ++i )
    {
      if ( SBp[i]->Tag == ch )
      {
        SpeedButtonClick(SBp[i]);
				break;
      }
    }
  }
}
//---------------------------------------------------------------------------
// Create a new ini file
//---------------------------------------------------------------------------
void __fastcall TFrmMain::CreateIniFile()
{
  TStringList* SL = new TStringList;

  SL->Add(";Configuration file for SS20Menu.exe");
  SL->Add("; Remove the semicolon and edit the names as needed");
	SL->Add("[Serial]");
	SL->Add("Comport=COM1");
	SL->Add("; Change to AltChar=Yes to use the ` rather than the ~ character");
	SL->Add("AltChar=No");
  SL->Add("");
  SL->Add("[SwitchNames]");
  SL->Add("A = Turn on switch  1");
  SL->Add("B = Turn on switch  2");
  SL->Add("C = Turn on switch  3");
  SL->Add("D = Turn on switch  4");
  SL->Add("E = Turn on switch  5");
  SL->Add("F = Turn on switch  6");
  SL->Add("G = Turn on switch  7");
  SL->Add("H = Turn on switch  8");
  SL->Add("I = Turn on switch  9");
  SL->Add("J = Turn on switch 10");
	SL->Add("K = Turn on switch 11");
	SL->Add("L = Turn on switch 12");
	SL->Add("M = Turn on switch 13");
	SL->Add("N = Turn on switch 14");
  SL->Add(";O = Turn on switch 15");
  SL->Add(";P = Turn on switch 16");
  SL->Add(";Q = Turn on switch 17");
  SL->Add(";R = Turn on switch 18");
  SL->Add(";S = Turn on switch 19");
  SL->Add(";T = Turn on switch 20");
  SL->SaveToFile(ChangeFileExt(Application->ExeName, ".ini"));
}
//---------------------------------------------------------------------------
// Shut off all ports
//---------------------------------------------------------------------------
void __fastcall TFrmMain::BtnAllOffClick(TObject *Sender)
{
  Screen->Cursor = crHourGlass;
  Application->ProcessMessages();
  VaComm->Baudrate=br9600;
  VaComm->Databits=db8;
  VaComm->Parity=paNone;
  VaComm->Stopbits=sb1;
  try
  {
    if ( !VaComm->Active() )
      VaComm->Open();
  }
  catch ( Exception &E )
  {
    Screen->Cursor = crDefault;
    Beep();
    Application->ShowException( &E);
    return;
  }
  if ( !VaComm->Active() )
  {
    Beep();
    Screen->Cursor = crDefault;
    ShowMessage("Cannot open "+VaComm->DeviceName);
    return;
  }
  VaComm->WriteChar(MenuChar);
  Sleep(500);
	VaComm->WriteChar(MenuChar);
  Sleep(50);
  VaComm->WriteChar('w');
  Sleep(50);
  VaComm->Close();
  Screen->Cursor = crDefault;
  Application->ProcessMessages();
}
//---------------------------------------------------------------------------
// If there are more than 12 buttons - put them in 2 columns
//---------------------------------------------------------------------------
void __fastcall TFrmMain::ArrangeButtons()
{
  int bottom = SBp[buttonCount-1]->Top + SBp[buttonCount-1]->Height;
  bool multCols = false;
  if ( buttonCount > 12 && SBp[buttonCount-1]->Width < Screen->Width /3 )
  {
    multCols = true;
    for ( int i = 0; i < ((buttonCount / 2)+1)  ; ++i )
    {
      if ( (!(buttonCount % 2)) && i == buttonCount/2 )
        break;
      SBp[i]->Left = 10;
      SBp[i]->Top = BUTTON_TOP;
      bottom = SBp[i]->Top + SBp[i]->Height;
      int num = i + buttonCount/2;
      int j = i;
      if ( (buttonCount % 2) ) // an odd number of buttons
        ++num;
      if ( SBp[num] != NULL )
      {
        SBp[num]->Left = SBp[j]->Left*2 + SBp[j]->Width;
        SBp[num]->Top = SBp[j]->Top;
      }
    }
  }

  BtnExit->Left = SBp[buttonCount-1]->Width + SBp[buttonCount-1]->Left - BtnExit->Width;
  if ( multCols )
    FrmMain->Width = SBp[11]->Left + SBp[11]->Width + SBp[0]->Left * 2;
  else
    FrmMain->Width = SBp[0]->Width + (SBp[0]->Left * 2 ) +10;

  bottom = bottom + 50;
  FrmMain->Height = bottom + BtnExit->Height+10;
  BtnExit->Top = bottom - BtnExit->Height - 10;
  BtnAllOff->Left = SBp[0]->Left;
  BtnAllOff->Top = bottom - BtnAllOff->Height - 10;
  // Prevent the buttons from overlapping on a narrow form
  while (BtnExit->Left < BtnAllOff->Left + BtnAllOff->Width + 5)
  {
  	BtnExit->Left += 2;
    BtnAllOff->Left -= 2;
  }
}
